@@ -118,6 +118,7 @@ struct ocp_sock {
 
 	/* for all listeners */
 	int lport;
+	char *bind_addr;
 	evconnlistener_cb listen_cb;
 
 	/* for port forwarding */
@@ -804,12 +805,25 @@ static void bind_all_listeners(void)
 	for (s = ocp_sock_bind_list; s; s = s->next) {
 		if (!s->listen_cb)
 			continue;
+		if (s->lport < 1 || s->lport > 65535)
+			die("invalid port number: %d\n", s->lport);
 
 		memset(&sock, 0, sizeof(sock));
-		sock.sin_family = AF_INET;
 		sock.sin_port = htons(s->lport);
-		sock.sin_addr.s_addr =
-			htonl(allow_remote ? INADDR_ANY : INADDR_LOOPBACK);
+		sock.sin_family = AF_INET;
+
+		if (s->bind_addr) {
+			/*
+			 * TODO: support IPv6 and multiple listening sockets
+			 * per hostname
+			 */
+			if (!inet_aton(s->bind_addr, &sock.sin_addr))
+				die("can't parse IP: '%s'\n", s->bind_addr);
+
+		} else {
+			sock.sin_addr.s_addr = htonl(allow_remote ?
+				INADDR_ANY : INADDR_LOOPBACK);
+		}
 
 		s->listener = evconnlistener_new_bind(event_base, s->listen_cb,
 			s, LEV_OPT_CLOSE_ON_FREE|LEV_OPT_REUSEABLE, -1,
@@ -860,6 +874,25 @@ bad:
 	die("Invalid port forward specifier: '%s'\n", opt);
 }
 
+static struct ocp_sock *dyn_fwd(const char *arg)
+{
+	struct ocp_sock *s;
+	const char *sep = strrchr(arg, ':');
+
+	if (sep) {
+		/* <addr>:<port> format */
+		s = new_listener(ocp_atoi(sep + 1), new_conn_cb);
+		s->bind_addr = xstrdup(arg);
+		s->bind_addr[sep - arg] = 0;
+	} else {
+		/* <port> only */
+		s = new_listener(ocp_atoi(arg), new_conn_cb);
+	}
+
+	s->conn_type = CONN_TYPE_SOCKS;
+	return s;
+}
+
 static struct option longopts[] = {
 	{ "ip",			1,	NULL,	'I' },
 	{ "mtu",		1,	NULL,	'M' },
@@ -931,8 +964,7 @@ int main(int argc, char **argv)
 			dns_domain = optarg;
 			break;
 		case 'D':
-			s = new_listener(ocp_atoi(optarg), new_conn_cb);
-			s->conn_type = CONN_TYPE_SOCKS;
+			s = dyn_fwd(optarg);
 			break;
 		case 'k':
 			keep_intvl = ocp_atoi(optarg);
